try:
    basestring
except NameError:
    # For python 3 compatibility
    basestring = str


def format_string(key):
    """
    Formats either measurement names, tag names or tag values.

    Measurement name and any optional tags separated by commas. Measurement names, tag keys,
    and tag values must escape any spaces, commas or equal signs using a backslash (\).
    For example: \ and \,.

    All tag values are stored as strings and should not be surrounded in quotes.
    """
    if isinstance(key, basestring):
        key = key.replace(",", "\,")
        key = key.replace(" ", "\ ")
        key = key.replace("=", "\=")
    return key


def format_value(value):
    """
    Integers are numeric values that do not include a decimal and are followed by a trailing i when inserted
    (e.g. 1i, 345i, 2015i, -10i). Note that all values must have a trailing i.
    If they do not they will be written as floats.

    Floats are numeric values that are not followed by a trailing i. (e.g. 1, 1.0, -3.14, 6.0e5, 10).

    Boolean values indicate true or false. Valid boolean strings for line protocol are
    (t, T, true, True, TRUE, f, F, false, False and FALSE).

    Strings are text values. All string field values must be surrounded in double-quotes ".
    If the string contains a double-quote, the double-quote must be escaped with a backslash, e.g. \".
    """
    if isinstance(value, basestring):
        value = value.replace('"', '\"')
        value = u'"{0}"'.format(value)
    elif isinstance(value, bool):
        value = str(value)
    elif isinstance(value, int):
        value = "{0}i".format(value)
    elif isinstance(value, float):
        value = str(value)
    return value



from __future__ import absolute_import
from django.conf import settings

from telegraf import defaults
from telegraf.client import TelegrafClient


telegraf = None

if telegraf is None:
    host = getattr(settings, 'TELEGRAF_HOST', defaults.HOST)
    port = getattr(settings, 'TELEGRAF_PORT', defaults.PORT)
    tags = getattr(settings, 'TELEGRAF_TAGS', defaults.TAGS)
    telegraf = TelegrafClient(host=host, port=port, tags=tags)



# -*- coding: utf-8 -*-

from telegraf.client import ClientBase, TelegrafClient, HttpClient
from telegraf.protocol import Line
from telegraf.utils import format_string, format_value
import unittest
import mock


class TestLine(unittest.TestCase):

    def test_format_key(self):
        self.assertEquals(format_string('foo'), 'foo')
        self.assertEquals(format_string('foo,bar'), 'foo\,bar')
        self.assertEquals(format_string('foo bar'), 'foo\ bar')
        self.assertEquals(format_string('foo ,bar'), 'foo\ \,bar')
        self.assertEquals(format_string('foo ,bar,baz=foobar'), 'foo\ \,bar\,baz\=foobar')

    def test_format_value(self):
        self.assertEquals(format_value('foo'), '"foo"')
        self.assertEquals(format_value('foo bar'), '"foo bar"')
        self.assertEquals(format_value('foo "and" bar'), '"foo \"and\" bar"')
        self.assertEquals(format_value(123), "123i")
        self.assertEquals(format_value(123.123), "123.123")
        self.assertEquals(format_value(True), "True")
        self.assertEquals(format_value(False), "False")

    def test_single_value(self):
        self.assertEquals(
            Line('some_series', 1).to_line_protocol(),
            'some_series value=1i'
        )

    def test_single_value_and_tags(self):
        self.assertEquals(
            Line('some_series', 1, {'foo': 'bar', 'foobar': 'baz'}).to_line_protocol(),
            'some_series,foo=bar,foobar=baz value=1i'
        )

    def test_multiple_values(self):
        self.assertEquals(
            Line('some_series', {'value': 232.123, 'value2': 123}).to_line_protocol(),
            'some_series value=232.123,value2=123i'
        )

    def test_multiple_values_and_tags(self):
        self.assertEquals(
            Line('some_series', {'value': 232.123, 'value2': 123}, {'foo': 'bar', 'foobar': 'baz'}).to_line_protocol(),
            'some_series,foo=bar,foobar=baz value=232.123,value2=123i'
        )

    def test_tags_and_measurement_with_whitespace_and_comma(self):
        self.assertEquals(
            Line(
                'white space',
                {'value, comma': "foo"},
                {'tag with, comma': 'hello, world'}
            ).to_line_protocol(),
            """white\ space,tag\ with\,\ comma=hello\,\ world value\,\ comma="foo\""""
        )

    def test_boolean_value(self):
        self.assertEquals(
            Line('some_series', True).to_line_protocol(),
            'some_series value=True'
        )

    def test_value_escaped_and_quoted(self):
        self.assertEquals(
            Line('some_series', 'foo "bar"').to_line_protocol(),
            'some_series value="foo \"bar\""'
        )

    def test_with_timestamp(self):
        self.assertEquals(
            Line('some_series', 1000, timestamp=1234134).to_line_protocol(),
            'some_series value=1000i 1234134'
        )

    def test_tags_ordered_properly(self):
        self.assertEquals(
            Line('some_series', 1, {'a': 1, 'baa': 1, 'AAA': 1, 'aaa': 1}).to_line_protocol(),
            'some_series,AAA=1,a=1,aaa=1,baa=1 value=1i'
        )

    def test_values_ordered_properly(self):
        self.assertEquals(
            Line('some_series', {'a': 1, 'baa': 1, 'AAA': 1, 'aaa': 1}).to_line_protocol(),
            'some_series AAA=1i,a=1i,aaa=1i,baa=1i'
        )


class TestClientBase(unittest.TestCase):

    def test_zero_value(self):
        self.client = ClientBase()
        self.client.send = mock.Mock()
        self.client.metric('some_series', 0)
        self.client.send.assert_called_with('some_series value=0i')

    def test_null_value(self):
        self.client = ClientBase()
        self.client.send = mock.Mock()
        self.client.metric('some_series', None)
        self.assertEqual(self.client.send.call_count, 0)

    def test_empty_values_dict(self):
        self.client = ClientBase()
        self.client.send = mock.Mock()
        self.client.metric('some_series', {})
        self.assertEqual(self.client.send.call_count, 0)

    def test_some_zero_values(self):
        self.client = ClientBase()
        self.client.send = mock.Mock()
        self.client.metric('some_series', {'value_one': 1, 'value_zero': 0, 'value_none': None})
        self.client.send.assert_called_with('some_series value_one=1i,value_zero=0i')


class TestTelegraf(unittest.TestCase):
    def setUp(self):
        self.host = 'host'
        self.port = 1234
        self.addr = (self.host, self.port)

    def test_sending_to_socket(self):
        self.client = TelegrafClient(self.host, self.port)
        self.client.socket = mock.Mock()

        self.client.metric('some_series', 1)
        self.client.socket.sendto.assert_called_with(b'some_series value=1i\n', self.addr)
        self.client.metric('cpu', {'value_int': 1}, {'host': 'server-01', 'region': 'us-west'})
        self.client.socket.sendto.assert_called_with(b'cpu,host=server-01,region=us-west value_int=1i\n', self.addr)

    def test_global_tags(self):
        self.client = TelegrafClient(self.host, self.port, tags={'host': 'host-001'})
        self.client.socket = mock.Mock()

        self.client.metric('some_series', 1)
        self.client.socket.sendto.assert_called_with(b'some_series,host=host-001 value=1i\n', self.addr)

        self.client.metric('some_series', 1, tags={'host': 'override-host-tag'})
        self.client.socket.sendto.assert_called_with(b'some_series,host=override-host-tag value=1i\n', self.addr)

    def test_utf8_encoding(self):
        self.client = TelegrafClient(self.host, self.port)
        self.client.socket = mock.Mock()

        self.client.metric(u'meäsurement', values={u'välue': 1, u'këy': u'valüe'}, tags={u'äpples': u'öranges'})
        self.client.socket.sendto.assert_called_with(
            b'me\xc3\xa4surement,\xc3\xa4pples=\xc3\xb6ranges k\xc3\xaby="val\xc3\xbce",v\xc3\xa4lue=1i\n', self.addr)


class TestTelegrafHttp(unittest.TestCase):
    def setUp(self):
        self.host = 'host'
        self.port = 1234
        self.url = 'http://{host}:{port}/write'.format(host=self.host, port=self.port)

    def test_sending_to_http(self):
        self.client = HttpClient(self.host, self.port)
        self.client.future_session = mock.Mock()

        self.client.metric('some_series', 1)
        self.client.future_session.post.assert_called_with(url=self.url, data='some_series value=1i')
        self.client.metric('cpu', {'value_int': 1}, {'host': 'server-01', 'region': 'us-west'})
        self.client.future_session.post.assert_called_with(url=self.url,
                                                           data='cpu,host=server-01,region=us-west value_int=1i')

    def test_global_tags(self):
        self.client = HttpClient(self.host, self.port, tags={'host': 'host-001'})
        self.client.future_session = mock.Mock()

        self.client.metric('some_series', 1)
        self.client.future_session.post.assert_called_with(data='some_series,host=host-001 value=1i', url=self.url)

        self.client.metric('some_series', 1, tags={'host': 'override-host-tag'})
        self.client.future_session.post.assert_called_with(data='some_series,host=override-host-tag value=1i',
                                                           url=self.url)



from telegraf.utils import format_string, format_value


class Line(object):
    def __init__(self, measurement, values, tags={}, timestamp=None):
        assert measurement, "Must have measurement"
        assert values not in (None, {}), "Must have values"

        # Name of the actual measurement
        self.measurement = measurement
        # Single value or a dict of value_name: value pairs
        self.values = values
        # Dict of tags if any
        self.tags = tags
        # User provided timestamp if any
        self.timestamp = timestamp

    def get_output_measurement(self):
        """
        Formats and escapes measurement name that can be rendered to line protocol
        """
        return format_string(self.measurement)

    def get_output_values(self):
        """
        Return an escaped string of comma separated value_name: value pairs
        """
        # Handle primitive values here and implicitly convert them to a dict because
        # it allows the API to be simpler.

        # Also influxDB mandates that each value also has a name so the default name
        # for any non-dict value is "value"
        if not isinstance(self.values, dict):
            metric_values = {'value': self.values}
        else:
            metric_values = self.values

        # Sort the values in lexicographically by value name
        sorted_values = sorted(metric_values.items())

        # Remove None values
        sorted_values = [(k, v) for k, v in sorted_values if v is not None]

        return u",".join(u"{0}={1}".format(format_string(k), format_value(v)) for k, v in sorted_values)

    def get_output_tags(self):
        """
        Return an escaped string of comma separated tag_name: tag_value pairs

        Tags should be sorted by key before being sent for best performance. The sort should
        match that from the Go bytes. Compare function (http://golang.org/pkg/bytes/#Compare).
        """

        # Sort the tags in lexicographically by tag name
        sorted_tags = sorted(self.tags.items())

        # Finally render, escape and return the tag string
        return u",".join(u"{0}={1}".format(format_string(k), format_string(v)) for k, v in sorted_tags)

    def get_output_timestamp(self):
        """
        Formats timestamp so it can be rendered to line protocol
        """
        return " {0}".format(self.timestamp) if self.timestamp else ""

    def to_line_protocol(self):
        """
        Converts the given metrics as a single line of InfluxDB line protocol
        """
        tags = self.get_output_tags()

        return u"{0}{1} {2}{3}".format(
            self.get_output_measurement(),
            "," + tags if tags else '',
            self.get_output_values(),
            self.get_output_timestamp()
        )



from abc import abstractmethod
from telegraf.protocol import Line
import socket


class ClientBase(object):

    def __init__(self, host='localhost', port=8094, tags=None):
        self.host = host
        self.port = port
        self.tags = tags or {}

    def metric(self, measurement_name, values, tags=None, timestamp=None):
        """
        Append global tags configured for the client to the tags given then
        converts the data into InfluxDB Line protocol and sends to to socket
        """
        if not measurement_name or values in (None, {}):
            # Don't try to send empty data
            return

        tags = tags or {}

        # Do a shallow merge of the metric tags and global tags
        all_tags = dict(self.tags, **tags)

        # Create a metric line from the input and then send it to socket
        line = Line(measurement_name, values, all_tags, timestamp)
        self.send(line.to_line_protocol())

    @abstractmethod
    def send(self, data):
        pass


class TelegrafClient(ClientBase):

    def __init__(self, host='localhost', port=8094, tags=None):
        super(TelegrafClient, self).__init__(host, port, tags)

        # Creating the socket immediately should be safe because it's UDP
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    def send(self, data):
        """
        Sends the given data to the socket via UDP
        """
        try:
            self.socket.sendto(data.encode('utf8') + b'\n', (self.host, self.port))
        except (socket.error, RuntimeError):
            # Socket errors should fail silently so they don't affect anything else
            pass


class HttpClient(ClientBase):

    def __init__(self, host='localhost', port=8094, tags=None):
        # only import HttpClient's dependencies if using HttpClient
        # if they're not found, inform the user how to install them
        try:
            from requests_futures.sessions import FuturesSession
        except ImportError:
            raise ImportError('pytelegraf[http] must be installed to use HTTP transport')

        super(HttpClient, self).__init__(host, port, tags)

        # the default url path for writing metrics to Telegraf is /write
        self.url = 'http://{host}:{port}/write'.format(host=self.host, port=self.port)

        # create a session to reuse the TCP socket when possible
        self.future_session = FuturesSession()

    def send(self, data):
        """
        Send the data in a separate thread via HTTP POST.
        HTTP introduces some overhead, so to avoid blocking the main thread,
        this issues the request in the background.
        """
        self.future_session.post(url=self.url, data=data)



from setuptools import find_packages, setup

setup(
    name='pytelegraf',
    version='0.3.1',
    description='Telegraf client',
    author='paksu',
    url='https://github.com/paksu/pytelegraf',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    extras_require={
        'http': ['requests-futures==0.9.7']
    },
    test_suite='telegraf.tests',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
